package cd.casic.ci.process.engine.scheduler.config;

import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.SchedulingConfigurer;
import org.springframework.scheduling.config.CronTask;
import org.springframework.scheduling.config.ScheduledTask;
import org.springframework.scheduling.config.ScheduledTaskRegistrar;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ScheduledFuture;

/**
 * @author HopeLi
 * @version v1.0
 * @ClassName PipelineSchedulerConfig
 * @Date: 2025/5/26 10:55
 * @Description:
 */

@Configuration
@EnableScheduling
@Slf4j
@Component("pipelineSchedulerConfig")
public class PipelineSchedulerConfig implements SchedulingConfigurer {
    private ScheduledTaskRegistrar taskRegistrar;

    private final Map<String, ScheduledTask> taskFutures = new HashMap<>();

    private final Map<String, CronTask> cronTaskMap = new HashMap<>();

    @Override
    public void configureTasks(@NotNull ScheduledTaskRegistrar taskRegistrar) {
        this.taskRegistrar = taskRegistrar;
        log.info("ScheduledTaskRegistrar 初始化完成");
    }


    public void addTask(String taskId, Runnable task, String cronExpression) {
        if (taskRegistrar == null) {
            log.error("【定时任务失败】taskRegistrar 尚未初始化，请检查调度器是否已启动");
            return;
        }

        CronTask cronTask = new CronTask(task, cronExpression);
        ScheduledTask scheduledTask = taskRegistrar.scheduleCronTask(cronTask);
        taskFutures.put(taskId, scheduledTask);
        cronTaskMap.put(taskId, cronTask);

        log.info("任务 [{}] 已添加，cron: {}", taskId, cronExpression);
    }

    public void removeTask(String taskId) {
        ScheduledTask scheduledTask = taskFutures.get(taskId);
        if (scheduledTask != null) {
            ScheduledFuture<?> future = getFuture(scheduledTask);
            if (future != null) {
                future.cancel(true);
            }
            taskFutures.remove(taskId);
            cronTaskMap.remove(taskId);

            log.info("任务 [{}] 已删除", taskId);
        }
    }

    public void updateTask(String taskId, String cronExpression) {
        removeTask(taskId);
        CronTask oldTask = cronTaskMap.get(taskId);
        if (oldTask == null) {
            throw new IllegalArgumentException("未找到对应的任务: " + taskId);
        }
        addTask(taskId, oldTask.getRunnable(), cronExpression);
        log.info("任务 [{}] 已更新，新 cron: {}", taskId, cronExpression);
    }

    public void startTask(String taskId) {
        ScheduledTask scheduledTask = taskFutures.get(taskId);
        if (scheduledTask == null) {
            log.warn("任务 [{}] 不存在", taskId);
            return;
        }

        ScheduledFuture<?> future = getFuture(scheduledTask);
        if (future != null && !future.isCancelled()) {
            log.info("任务 [{}] 正在运行中，无需重启", taskId);
            return;
        }

        CronTask cronTask = cronTaskMap.get(taskId);
        if (cronTask == null) {
            log.warn("未找到对应的任务定义: {}", taskId);
            return;
        }

        ScheduledTask newTask = taskRegistrar.scheduleCronTask(cronTask);
        taskFutures.put(taskId, newTask);
        log.info("任务 [{}] 已重启", taskId);
    }

    /**
     * 停止一个任务
     */
    public void stopTask(String taskId) {
        removeTask(taskId);
        log.info("任务 [{}] 已停止", taskId);
    }


    /**
     * 获取 ScheduledTask 中的 future 字段
     */
    private ScheduledFuture<?> getFuture(ScheduledTask scheduledTask) {
        try {
            Field field = scheduledTask.getClass().getDeclaredField("future");
            field.setAccessible(true);
            return (ScheduledFuture<?>) field.get(scheduledTask);
        } catch (Exception e) {
            log.error("反射获取 future 失败: {}", e.getMessage());
            return null;
        }
    }


    /**
     * 判断任务是否已取消
     */
    public boolean isTaskCancelled(String taskId) {
        ScheduledTask scheduledTask = taskFutures.get(taskId);
        if (scheduledTask == null) return true;

        ScheduledFuture<?> future = getFuture(scheduledTask);
        return future != null && future.isCancelled();
    }

//    @Component
//    public static class PipelineSchedulerTask implements Runnable {
//
//        @Override
//        public void run() {
//            // 定时任务逻辑代码
//            System.out.println("Pipeline Task: " + new Date());
//        }
//    }
}
